/**
 * @file
 * Drupal Component Builder behaviors.
 */
(function (Drupal) {

  'use strict';

  /**
   * Update the URL with the given params.
   * @param params
   * @return {string}
   */
  function updateUrlParams(params) {
    const urlObj = new URL(window.location.href);
    const searchParams = new URLSearchParams(urlObj.search);

    for (const key in params) {
      searchParams.set(key, params[key]);
    }

    urlObj.search = searchParams.toString();
    return urlObj.toString();
  }

  /**
   * Remove the given query param from the URL.
   * @param paramName
   * @return {string}
   */
  function removeQueryParam(paramName) {
    const url = new URL(window.location.href);
    url.searchParams.delete(paramName);
    const newUrl = url.toString();
    window.history.pushState({ path: newUrl }, "", newUrl);
  }

  /**
   * Add a behavior to reload the page when the user presses CTRL + R.
   */
  Drupal.behaviors.reloadBehavior = {
    attach(context) {
      const cacheClearForm = context.querySelector("#cache-clear-form");

      if (!cacheClearForm) {
        return;
      }

      window.addEventListener("keydown", (event) => {
        if (
          ((event.ctrlKey || event.metaKey) && event.key === "r") ||
          event.keyCode === 116
        ) {
          event.preventDefault();

          console.log("Clearing cache...");
          cacheClearForm.submit();
        }
      });
    },
  };

  /**
   * Add a behavior to the iframe resize select.
   */
  Drupal.behaviors.resizeIframe = {
    attach(context, settings) {
      const iframeResizeSelect = context.querySelector("#iframe-resize-select");
      const iframeContainer = context.querySelector(".dab-container");
      const iframe = context.querySelector(".dab-iframe");

      if (!iframeResizeSelect || !iframeContainer || !iframe) {
        return;
      }

      const changeIframeWidth = (value) => {
        const newUrl = updateUrlParams({ "iframe-width": value });
        window.history.pushState({ path: newUrl }, "", newUrl);

        switch (value) {
          case "mobile":
            iframeContainer.style.width = "320px";
            break;

          case "tablet":
            iframeContainer.style.width = "768px";
            break;

          case "desktop":
            iframeContainer.style.width = "1024px";
            break;

          default:
          case "reset":
            iframeContainer.style.width = "";
            removeQueryParam("iframe-width");
            break;
        }
      };

      changeIframeWidth(iframeResizeSelect.value);

      iframeResizeSelect.addEventListener("change", (event) => {
        const { value } = event.target;
        changeIframeWidth(value);
      });
    },
  };

  /**
   * Add a behavior to the iframe provider select.
   */
  Drupal.behaviors.templateSelection = {
    attach(context, settings) {
      const templateSelect = context.querySelector("#template-select");
      const pathName = window.location.pathname.split("/");

      if (!templateSelect) {
        return;
      }

      templateSelect.addEventListener("change", (event) => {
        const providerValue = event.target.value;
        pathName[5] = providerValue;
        const fullUrl = pathName.join("/");

        if (fullUrl !== window.location.pathname) {
          window.location.pathname = fullUrl;
        }
      });
    },
  };

  /**
   * Add a behavior to the iframe version select.
   */
  Drupal.behaviors.versionSelection = {
    attach(context, settings) {
      const versionSelect = context.querySelector("#version-select");

      if (!versionSelect) {
        return;
      }

      versionSelect.addEventListener("change", (event) => {
        const versionValue = event.target.value;

        if (versionValue === "") {
          removeQueryParam("version");
          window.location.reload();
        } else {
          const fullUrl = updateUrlParams({ version: versionValue });
          window.location.href = fullUrl;
        }
      });
    },
  };

  /**
   * Reset all parameters in URL.
   */
  Drupal.behaviors.resetParams = {
    attach(context, settings) {
      const versionSelect = context.querySelector("#reset-query-button");

      if (!versionSelect) {
        return;
      }

      if (window.location.search !== "") {
        versionSelect.style.display = "block";
      } else {
        versionSelect.style.display = "none";
      }
    },
  };
})(Drupal, drupalSettings);
